searchNodes=[{"ref":"Argon2.html","title":"Argon2","type":"module","doc":"Elixir wrapper for the Argon2 password hashing function. This module implements the Comeonin and Comeonin.PasswordHash behaviours, providing the following functions: add_hash/2 - takes a password as input and returns a map containing the password hash check_pass/3 - takes a user struct and password as input and verifies the password no_user_verify/1 - runs the hash function, but always returns false hash_pwd_salt/2 - hashes the password with a randomly-generated salt verify_pass/2 - verifies a password For a lower-level API, see Argon2.Base. Configuration See the documentation for Argon2.Stats for information about configuration. Options In addition to the options listed below, the add_hash, no_user_verify and hash_pwd_salt functions all take options that are then passed on to the hash_password function in the Argon2.Base module. See the documentation for Argon2.Base.hash_password/3 for details. add_hash hash_key - the key used in the map for the password hash the default is password_hash :salt_len - the length of the random salt the default is 16 (the minimum is 8) bytes check_pass hash_key - the key used in the user struct for the password hash if this is not set, check_pass will look for password_hash, and then encrypted_password hide_user - run no_user_verify to prevent user enumeration the default is true set this to false if you do not want to hide usernames hash_pwd_salt :salt_len - the length of the random salt the default is 16 (the minimum is 8) bytes Examples The following examples show how to hash a password with a randomly-generated salt and then verify a password: iex&gt; hash = Argon2.hash_pwd_salt(&quot;password&quot;) ...&gt; Argon2.verify_pass(&quot;password&quot;, hash) true iex&gt; hash = Argon2.hash_pwd_salt(&quot;password&quot;) ...&gt; Argon2.verify_pass(&quot;incorrect&quot;, hash) false add_hash The put_pass_hash function below is an example of how you can use add_hash to add the password hash to the Ecto changeset. defp put_pass_hash(%Ecto.Changeset{valid?: true, changes: %{password: password}} = changeset) do change(changeset, Argon2.add_hash(password)) end defp put_pass_hash(changeset), do: changeset This function will return a changeset with %{password_hash: password_hash, password: nil} added to the changes map. check_pass The following is an example of calling this function with no options: def verify_user(%{&quot;password&quot; =&gt; password} = params) do params |&gt; Accounts.get_by() |&gt; Argon2.check_pass(password) end The Accounts.get_by function in this example takes the user parameters (for example, email and password) as input and returns a user struct or nil. Argon2 Argon2 is the winner of the Password Hashing Competition (PHC). Argon2 is a memory-hard password hashing function which can be used to hash passwords for credential storage, key derivation, or other applications. Argon2 has the following three variants (Argon2id is the default): Argon2d - suitable for applications with no threats from side-channel timing attacks (eg. cryptocurrencies) Argon2i - suitable for password hashing and password-based key derivation Argon2id - a hybrid of Argon2d and Argon2i Argon2i, Argon2d, and Argon2id are parametrized by: A time cost, which defines the amount of computation realized and therefore the execution time, given in number of iterations A memory cost, which defines the memory usage, given in kibibytes A parallelism degree, which defines the number of parallel threads More information can be found in the documentation for the Argon2.Stats module and at the Argon2 reference C implementation repository. Comparison with Bcrypt / Pbkdf2 Currently, the most popular password hashing functions are Bcrypt, which was presented in 1999, and Pbkdf2 (pbkdf2_sha256 or pbkdf2_sha512), which dates back to 2000. Both are strong password hashing functions with no known vulnerabilities, and their algorithms have been used and widely reviewed for over 10 years. To help you decide whether you should use Argon2 instead, here is a brief comparison of Bcrypt / Pbkdf2 with Argon2. Argon2 is a lot newer, and this can be considered to be both an advantage and a disadvantage. On the one hand, Argon2 benefits from more recent research, and it is designed to combat the kinds of attacks which have become more common over the past decade, such as the use of GPUs or dedicated hardware. On the other hand, Argon2 has not received the same amount of scrutiny that Bcrypt / Pbkdf2 has. One of the main differences is that Argon2 is a memory-hard function, and this means that it is designed to use a lot more memory than Bcrypt / Pbkdf2. With Bcrypt / Pbkdf2, attackers can use GPUs to hash several hundred / thousand passwords in parallel. This can result in significant gains in the time it takes an attacker to crack passwords. Argon2&#39;s memory cost means that it is a lot more difficult for attackers to benefit from using GPUs or other dedicated hardware."},{"ref":"Argon2.html#gen_salt/1","title":"Argon2.gen_salt/1","type":"function","doc":"Generate a random salt. The default length for the salt is 16 bytes. We do not recommend using a salt shorter than the default."},{"ref":"Argon2.Base.html","title":"Argon2.Base","type":"module","doc":"Lower-level api for Argon2. These functions can be useful if you want more control over some of the options. In most cases, you will not need to call these functions directly."},{"ref":"Argon2.Base.html#encodedlen_nif/6","title":"Argon2.Base.encodedlen_nif/6","type":"function","doc":"Calculate the length of the encoded hash."},{"ref":"Argon2.Base.html#error_nif/1","title":"Argon2.Base.error_nif/1","type":"function","doc":"Translate the error code to an error message."},{"ref":"Argon2.Base.html#hash_nif/10","title":"Argon2.Base.hash_nif/10","type":"function","doc":"Hash a password using Argon2."},{"ref":"Argon2.Base.html#hash_password/3","title":"Argon2.Base.hash_password/3","type":"function","doc":"Hash a password using Argon2. Options There are six options: :t_cost - time cost :m_cost - memory usage :parallelism - number of parallel threads :format - output format this value can be :encoded - encoded with Argon2 crypt format :raw_hash - raw hash output in hexadecimal format :report - raw hash and encoded hash, together with the options used :encoded is the default :hashlen - length of the hash (in bytes) the default is 32 :argon2_type - Argon2 type this value should be 0 (Argon2d), 1 (Argon2i) or 2 (Argon2id) the default is 2 (Argon2id) The t_cost, m_cost, parallelism and argon2_type can also be set in the config. See the documentation for Argon2.Stats for more information about choosing these values. Examples The following example changes the default t_cost and m_cost: Argon2.Base.hash_password(&quot;password&quot;, &quot;somesaltSOMESALT&quot;, [t_cost: 4, m_cost: 18]) In the example below, the Argon2 type is changed to Argon2d: Argon2.Base.hash_password(&quot;password&quot;, &quot;somesaltSOMESALT&quot;, [argon2_type: 0]) To use Argon2i, use argon2_type: 1."},{"ref":"Argon2.Base.html#init/0","title":"Argon2.Base.init/0","type":"function","doc":""},{"ref":"Argon2.Base.html#verify_nif/3","title":"Argon2.Base.verify_nif/3","type":"function","doc":"Verify a password using Argon2."},{"ref":"Argon2.Stats.html","title":"Argon2.Stats","type":"module","doc":"Module to provide statistics for the Argon2 password hashing function. The report/1 function in this module can be used to help you configure Argon2. Configuration The following four parameters can be set in the config file (these can all be overridden using keyword options): t_cost - time cost the amount of computation, given in number of iterations 8 is the default m_cost - memory usage 17 is the default - this will produce a memory usage of 128 MiB (2 ^ 17 KiB) parallelism - number of parallel threads 4 is the default argon2_type - argon2 variant to use 0 (Argon2d), 1 (Argon2i) or 2 (Argon2id) 2 is the default (Argon2id) It is important to note that the default values are not necessarily the recommended values. A lot will depend on what hardware you are using and how much time you want the hashing function to take. See the Choosing parameters section for more details. Test values The following values can be used to speed up tests. config :argon2_elixir, t_cost: 1, m_cost: 8 NB. do not use these values in production. Choosing parameters The following is a guide on how to choose the parameters. Decide which type to use. If you do not know the difference between the types, choose Argon2id. Decide how many threads to use - the parallelism option. Decide how much memory each call should use - m_cost. Decide how much time each call should take. a. The Argon2 draft guidelines recommend 500 milliseconds (0.5 seconds). Run the function using the values chosen in steps 1-3. Work out the maximum t_cost (number of iterations) such that the running time does not exceed the value chosen in step 4. If the time taken exceeds the value chosen in step 4 even for one iteration, reduce the m_cost accordingly."},{"ref":"Argon2.Stats.html#report/1","title":"Argon2.Stats.report/1","type":"function","doc":"Hash a password with Argon2 and print out a report. This function hashes the password and salt with Argon2.Base.hash_password/3 and prints out statistics which can help you choose how to configure Argon2. Options In addition to the options for Argon2.Base.hash_password/3, there are also the following options: :password - the password used the default is &quot;password&quot; :salt - the salt used the default is &quot;somesaltSOMESALT&quot;"}]